@@ -252,7 +252,7 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
         }
         rs_log_info("no visible output file, going to add \"-o %s\" at end",
                       ofile);
-        dcc_argv_append(argv, "-o");
+        dcc_argv_append(argv, strdup("-o"));
         dcc_argv_append(argv, ofile);
         *output_file = ofile;
     }
@@ -346,7 +346,7 @@ int dcc_deepcopy_argv(char **from, /*@-nullstate@*/ char ***out)
  * Used to change "-c" or "-S" to "-E", so that we get preprocessed
  * source.
  **/
-int dcc_set_action_opt(char **a, char *new_c)
+int dcc_set_action_opt(char **a, const char *new_c)
 {
     int gotone = 0;
     
@@ -419,8 +419,11 @@ int dcc_set_output(char **a, char *ofname)
 int dcc_set_input(char **a, char *ifname)
 {
     int i;
-    
-    rs_trace("command before: %s", dcc_argv_tostr(a));
+    char *astr;
+
+    astr = dcc_argv_tostr(a);
+    rs_trace("command before: %s", astr);
+    free(astr);
 
     for (i =0; a[i]; i++)
         if (dcc_is_source(a[i])) {
