@@ -1,9 +1,8 @@
-/* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
+/* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78; -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/bulk.c,v 1.23 2002/11/01 07:58:31 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -50,12 +49,21 @@
 #include <errno.h>
 
 #include <sys/stat.h>
+#include <sys/time.h>
 
 #include "distcc.h"
 #include "trace.h"
 #include "io.h"
 #include "rpc.h"
 #include "bulk.h"
+#include "time.h"
+
+#include "timeval.h"
+
+#ifndef O_BINARY
+#  define O_BINARY 0
+#endif
+
 
 /**
  * Open a file for read, and also put its size into @p fsize.
@@ -78,7 +86,7 @@ int dcc_open_read(const char *fname, int *ifd, off_t *fsize)
 {
     struct stat buf;
     
-    *ifd = open(fname, O_RDONLY);
+    *ifd = open(fname, O_RDONLY|O_BINARY);
     if (*ifd == -1) {
         int save_errno = errno;
         if (save_errno == ENOENT) {
@@ -103,6 +111,55 @@ int dcc_open_read(const char *fname, int *ifd, off_t *fsize)
 }
 
 
+static void dcc_calc_rate(size_t size_out,
+                          struct timeval *before,
+                          struct timeval *after,
+                          double *secs,
+                          double *rate)
+{
+    struct timeval delta;
+    
+    timeval_subtract(&delta, after, before);
+
+    *secs = (double) delta.tv_sec + (double) delta.tv_usec / 1e6;
+
+    *rate = ((double) size_out / *secs) / 1024.0;
+}
+
+
+/**
+ * Transmit a file and print timing statistics.  Only used for big files.
+ *
+ * Wrapper around dcc_x_file(). 
+ **/
+int dcc_x_file_timed(int ofd, const char *fname, const char *token,
+                     size_t *size_out)
+{
+    struct timeval before, after;
+    int ret;
+    size_t size;
+
+    if (gettimeofday(&before, NULL))
+        rs_log_warning("gettimeofday failed");
+
+    ret = dcc_x_file(ofd, fname, token, &size);
+    if (size_out)
+        *size_out = size;           /* separate to allow for size_out NULL */
+
+    if (gettimeofday(&after, NULL)) {
+        rs_log_warning("gettimeofday failed");
+    } else {
+        double secs, rate;
+        
+        dcc_calc_rate(size, &before, &after, &secs, &rate);
+        rs_log_info("%ld bytes sent in %.3fs, rate %.0fkB/s",
+                    (long) size, secs, rate);
+    }
+
+    return ret;
+}
+
+
 /**
  * Transmit from a local file to the network.
  *
@@ -170,7 +227,7 @@ int dcc_r_file(int ifd, const char *filename, size_t len)
         return -1;
     }
 
-    ofd = open(filename, O_TRUNC|O_WRONLY|O_CREAT|O_EXCL, 0666);
+    ofd = open(filename, O_TRUNC|O_WRONLY|O_CREAT|O_EXCL|O_BINARY, 0666);
     if (ofd == -1) {
         rs_log_error("failed to create %s: %s", filename, strerror(errno));
         return -1;
@@ -193,42 +250,6 @@ int dcc_r_file(int ifd, const char *filename, size_t len)
 }
 
 
-/**
- * Receive a file stream from the network into a local fifo, which
- * must already exist.
- *
- * The fifo is closed when we either reach the end of the network
- * body, or fail.
- **/
-int dcc_r_fifo(int ifd, const char *fifo_name, size_t len)
-{
-    int ofd, ret, close_ret;
-
-    /* This blocks until the child starts */
-    ofd = open(fifo_name, O_WRONLY, 0600);
-    if (ofd == -1 && errno == EINTR) {
-        rs_log_notice("interrupted while opening fifo %s; perhaps command exited without reading it?",
-                      fifo_name);
-        return -1;
-    } else if (ofd == -1) {
-        rs_log_warning("failed to open fifo %s: %s", fifo_name,
-                       strerror(errno));
-        return -1;
-    }
-
-    /* Removing the fifo wouldn't help because the child has already
-     * started reading */
-    ret = dcc_r_file_body(ofd, ifd, len);
-    close_ret = dcc_close(ofd);
-
-    if (!ret && !close_ret) {
-        rs_log_info("received %d bytes to fifo %s", len, fifo_name);
-    }
-    
-    return ret ? ret : close_ret;
-}
-
-
 /**
  * Common routine for reading a file body used by both fifos and
  * regular files.
@@ -240,3 +261,32 @@ int dcc_r_file_body(int ofd, int ifd, size_t len)
 {
     return dcc_pump_readwrite(ofd, ifd, (size_t) len);
 }
+
+
+/**
+ * Receive a file and print timing statistics.  Only used for big files.
+ *
+ * Wrapper around dcc_r_file(). 
+ **/
+int dcc_r_file_timed(int ifd, const char *fname, size_t size)
+{
+    struct timeval before, after;
+    int ret;
+
+    if (gettimeofday(&before, NULL))
+        rs_log_warning("gettimeofday failed");
+
+    ret = dcc_r_file(ifd, fname, size);
+
+    if (gettimeofday(&after, NULL)) {
+        rs_log_warning("gettimeofday failed");
+    } else {
+        double secs, rate;
+        
+        dcc_calc_rate(size, &before, &after, &secs, &rate);
+        rs_log_info("%ld bytes received in %.3fs, rate %.0fkB/s",
+                    (long) size, secs, rate);
+    }
+
+    return ret;
+}
