@@ -1,7 +1,7 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78 -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/cpp.c,v 1.4 2002/11/21 07:53:47 mbp Exp $ 
+ * $Header: /data/cvs/distcc/src/cpp.c,v 1.5 2003/02/22 01:41:31 mbp Exp $ 
  *
  * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
  *
@@ -109,7 +109,7 @@ int dcc_cpp_maybe(char **argv, char *input_fname, char **cpp_fname,
         || (ret = dcc_set_action_opt(cpp_argv, "-E")))
         return ret;
 
-    return dcc_spawn_child("cpp", cpp_argv, cpp_pid,
+    return dcc_spawn_child(cpp_argv, cpp_pid,
                            "/dev/null", *cpp_fname, NULL);
 }
 
