@@ -1,7 +1,6 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78 -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/distcc.c,v 1.143 2003/01/27 12:13:06 mbp Exp $ 
  *
  * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
@@ -36,7 +35,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -61,6 +59,7 @@
 #include "where.h"
 #include "lock.h"
 #include "cpp.h"
+#include "clirpc.h"
 
 
 /* Name of this program, for trace.c */
@@ -126,143 +125,6 @@ static void dcc_show_usage(void)
 }
 
 
-/*
- * Transmit header for whole request.
- */
-static int dcc_x_req_header(int fd)
-{
-    return dcc_x_token_int(fd, "DIST", PROTO_VER);
-}
-
-
-
-static int dcc_x_argv(int fd, char **argv)
-{
-    int i;
-    int argc;
-    
-    argc = dcc_argv_len(argv);
-    
-    if (dcc_x_token_int(fd, "ARGC", argc))
-        return -1;
-    
-    for (i = 0; i < argc; i++) {
-        if (dcc_x_token_int(fd, "ARGV", (unsigned) strlen(argv[i])))
-            return -1;
-        if (dcc_write_str(fd, argv[i]))
-            return -1;
-    }
-
-    return 0;
-}
-
-
-static int dcc_send_job(int net_fd,
-                        char **argv,
-                        pid_t cpp_pid,
-                        int *status,
-                        const char *cpp_fname)
-{
-    long stime_usec, utime_usec;
-    int ret;
-
-    if ((ret = dcc_x_req_header(net_fd))
-        || (ret = dcc_x_argv(net_fd, argv)))
-        return ret;
-
-    if (cpp_pid) {
-        /* Wait for cpp to finish (if not already done), check the
-         * result, then send the .i file */
-        
-        if ((ret = dcc_collect_child(cpp_pid, status, &utime_usec, &stime_usec))
-            || (ret = dcc_report_rusage("cpp", utime_usec, stime_usec)))
-            return ret;
-
-        /* Although cpp failed, there is no need to try running the command
-         * locally, because we'd presumably get the same result.  Therefore
-         * critique the command and log a message and return an indication
-         * that compilation is complete. */
-        if (dcc_critique_status(*status, "cpp", dcc_gethostname()))
-            return 0;
-    }
-
-    /* TODO: Add test case for this marginal case: cpp returns 0, but does not
-     * create an output file. */
-    if (access(cpp_fname, R_OK) != 0) {
-        rs_log_error("can't read cpp output \"%s\": %s",
-                     cpp_fname, strerror(errno));
-        return EXIT_IO_ERROR;
-    }
-
-    if ((ret = dcc_x_file(net_fd, cpp_fname, "DOTI", NULL)))
-        return ret;
-
-    rs_trace("client finished sending request to server");
-
-    return 0;
-}
-
-
-/**
- * Send job with socket corked.
- *
- * Make sure to uncork on return.  For success, this is necessary to
- * make sure the whole request gets pushed to the other side quickly.
- *
- * For failure, we need to uncork the socket to work around a bug in
- * Linux 2.2 that causes the socket to get stuck in FIN_WAIT1 if it is
- * closed while corked.  Unfortunately, this is not a full solution:
- * if distcc crashes or is killed the same situation will pertain.
- *
- * http://marc.theaimsgroup.com/?l=linux-netdev&r=1&b=200209&w=2
- **/
-static int dcc_send_job_corked(int net_fd,
-                               char **argv,
-                               pid_t cpp_pid,
-                               int *status,
-                               const char *cpp_fname)
-{
-    int ret;
-
-    tcp_cork_sock(net_fd, 1);
-
-    ret = dcc_send_job(net_fd, argv, cpp_pid, status, cpp_fname);
-    
-    tcp_cork_sock(net_fd, 0);
-    
-    return ret;
-}
-
-
-static int dcc_retrieve_results(int net_fd, int *status, const char *output_fname)
-{
-    int len;
-    int ret;
-    int o_len;
-
-    if ((ret = dcc_r_result_header(net_fd))
-        || (ret = dcc_r_cc_status(net_fd, status))
-        || (ret = dcc_r_token_int(net_fd, "SERR", &len))
-        || (ret = dcc_r_file_body(STDERR_FILENO, net_fd, len))
-        || (ret = dcc_r_token_int(net_fd, "SOUT", &len))
-        || (ret = dcc_r_file_body(STDOUT_FILENO, net_fd, len))
-        || (ret = dcc_r_token_int(net_fd, "DOTO", &o_len)))
-        return ret;
-
-    /* Previously we would skip retrieving the .o file unless the compiler
-     * completed successfully, but it seems cleaner to have the protocol
-     * always the same and to always drain the network.  The server always
-     * sends an 0 byte file on failure anyhow. */
-
-    if (o_len)
-        return dcc_r_file(net_fd, output_fname, o_len);
-    else {
-        rs_log_notice("skipping retrieval of 0 byte object file %s",
-                      output_fname);
-        return 0;
-    }
-}
-
 
 /**
  * Pass a compilation across the network.
@@ -291,7 +153,7 @@ static int dcc_retrieve_results(int net_fd, int *status, const char *output_fnam
  * @param status on return contains the wait-status of the remote
  * compiler.
  *
- * @return 0 on success, otherwise error.  Returning nonzero does not
+ * @returns 0 on success, otherwise error.  Returning nonzero does not
  * necessarily imply the remote compiler itself succeeded, only that
  * there were no communications problems. 
  **/
@@ -299,7 +161,8 @@ static int dcc_compile_remote(char **argv,
                               char *cpp_fname, char *output_fname,
                               pid_t cpp_pid,
                               const struct dcc_hostdef *host,
-                              int *status)
+                              int *status,
+                              int xmit_lock_fd)
 {
     int net_fd;
     int ret;
@@ -314,6 +177,9 @@ static int dcc_compile_remote(char **argv,
      * the connection will have been dropped and we need not bother trying to
      * get any response from the server. */
     ret = dcc_send_job_corked(net_fd, argv, cpp_pid, status, cpp_fname);
+
+    dcc_unlock(xmit_lock_fd);
+    
     if (ret == 0 && *status == 0) {
         ret = dcc_retrieve_results(net_fd, status, output_fname);
     }
@@ -339,6 +205,8 @@ static int dcc_compile_remote(char **argv,
  * schedule, etc another process.  The problem is that in that case we
  * can't clean up our temporary files, and (not so important) we can't
  * log our resource usage.
+ *
+ * This is called with a lock on localhost already held.
  **/
 static int dcc_compile_local(char *argv[])
 {
@@ -346,23 +214,13 @@ static int dcc_compile_local(char *argv[])
     int ret;
     int status;
     long u_us, s_us;
-    
-    char *buildhost = "localhost";
-
-    /* FIXME: This has the problem that if we originally want to build on some
-     * other machine, but then fall back to building locally (e.g. because of
-     * network error), the client will run with both local and remote locks
-     * held.  That shouldn't cause problems, but it will skew load balancing.
-     * Probably not a problem since the remote machine is presumably having
-     * trouble anyhow. */
-
-    dcc_lock_local("compiler");
+    const char *buildhost = "localhost";
 
     dcc_note_execution(buildhost, argv);
 
     /* We don't do any redirection of file descriptors when running locally,
      * so if for example cpp is being used in a pipeline we should be fine. */
-    if ((ret = dcc_spawn_child("compile", argv, &pid, NULL, NULL, NULL)) != 0)
+    if ((ret = dcc_spawn_child(argv, &pid, NULL, NULL, NULL)) != 0)
         return ret;
 
     if ((ret = dcc_collect_child(pid, &status, &u_us, &s_us)))
@@ -400,17 +258,21 @@ static int dcc_build_somewhere(char *argv[], int *status)
     char *input_fname, *output_fname, *cpp_fname;
     char **argv_stripped;
     pid_t cpp_pid = 0;
+    int xmit_lock_fd, cpu_lock_fd;
     int ret;
     struct dcc_hostdef *host = NULL;
 
+    /* TODO: Perhaps tidy up these gotos. */
+
     if (dcc_scan_args(argv, &input_fname, &output_fname, &argv) != 0) {
         /* we need to scan the arguments even if we already know it's
          * local, so that we can pick up distcc client options. */
-        goto run_local;
+        goto lock_local;
     }
 
-    if ((ret = dcc_pick_buildhost("compile", &host)) != 0) {
-        /* doesn't happen at the moment */
+    if ((ret = dcc_pick_host_from_env(&host, &xmit_lock_fd, &cpu_lock_fd)) != 0) {
+        /* Doesn't happen at the moment: all failures are masked by
+           returning localhost. */
         goto fallback;
     }
 
@@ -424,19 +286,25 @@ static int dcc_build_somewhere(char *argv[], int *status)
         goto fallback;
 
     if ((ret = dcc_compile_remote(argv_stripped, cpp_fname, output_fname,
-                                  cpp_pid, host, status)) != 0) {
+                                  cpp_pid, host, status, xmit_lock_fd)) != 0) {
         /* Returns zero if we successfully ran the compiler, even if
          * the compiler itself bombed out. */
         goto fallback;
     }
+
+    dcc_unlock(cpu_lock_fd);
         
-    return dcc_critique_status(*status, argv[0], host->hostname);
+    return dcc_critique_status(*status, "compile", host->hostname);
 
   fallback:
     /* "You guys are so lazy!  Do I have to do all the work myself??" */
     rs_log_warning("failed to distribute to \"%s\", running locally instead",
                    host && host->hostname ? host->hostname : "(unknown)");
-    
+
+  lock_local:
+
+    dcc_lock_local(&xmit_lock_fd, &cpu_lock_fd);
+
   run_local:
     return dcc_compile_local(argv);
 }
