@@ -1,7 +1,6 @@
-/* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
+/* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78; -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/dopt.c,v 1.29 2003/01/27 12:06:42 mbp Exp $ 
  *
  * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
@@ -39,12 +38,17 @@
 #include <errno.h>
 #include <popt.h>
 
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+
 #include "distcc.h"
 #include "trace.h"
 #include "io.h"
 #include "opt.h"
 #include "exitcode.h"
 #include "setuid.h"
+#include "access.h"
 
 int arg_nice_inc;
 int arg_nchildren = 2;          /**< Number of children running jobs
@@ -60,6 +64,8 @@ int opt_daemon_mode = 0;
 int opt_inetd_mode = 0;
 int opt_no_fifo = 0;
 
+struct dcc_allow_list *opt_allowed = NULL;
+
 /**
  * If true, don't detach from the parent.  This is probably necessary
  * for use with daemontools or other monitoring programs, and is also
@@ -78,6 +84,7 @@ enum {
 
 
 const struct poptOption options[] = {
+    { "allow", 'a',      POPT_ARG_STRING, 0, 'a', 0, 0 },
     { "concurrent", 'n', POPT_ARG_INT, &arg_nchildren, 'n', 0, 0 },
     { "daemon", 0,       POPT_ARG_NONE, &opt_daemon_mode, 0, 0, 0 },
     { "help", 0,         POPT_ARG_NONE, 0, '?', 0, 0 },
@@ -112,6 +119,7 @@ static void distccd_show_usage(void)
 "    -P, --pid-file FILE        save daemon process id to file\n"
 "    -N, --nice LEVEL           lower priority, 20=most nice\n"
 "    --user USER                if run by root, change to this persona\n"
+"    -a, --allow IP[/MASK]      client address access control\n"
 "  Debug and trace:\n"
 "    --verbose                  include debug messages in log\n"
 "    --no-fork                  single process only (for debugging ONLY)\n"
@@ -123,11 +131,9 @@ static void distccd_show_usage(void)
 "    --daemon                   bind and listen on socket\n"
 "\n"
 "distccd runs either from inetd or as a standalone daemon to compile\n"
-"files submitted by the distcc client.  distccd runs on TCP port %d"
-"by default.\n"
+"files submitted by the distcc client.\n"
 "\n"
-"distccd should only run on trusted networks.\n",
-    DISTCC_DEFAULT_PORT
+"distccd should only run on trusted networks.\n"
 );
 }
 
@@ -135,17 +141,31 @@ static void distccd_show_usage(void)
 int distccd_parse_options(int argc, const char **argv)
 {
     poptContext po;
-    int rc, exitcode;
+    int po_err, exitcode;
 
     po = poptGetContext("distccd", argc, argv, options, 0);
 
-    while ((rc = poptGetNextOpt(po)) != -1) {
-        switch (rc) {
+    while ((po_err = poptGetNextOpt(po)) != -1) {
+        switch (po_err) {
         case '?':
             distccd_show_usage();
             exitcode = 0;
             goto out_exit;
 
+        case 'a': {
+            /* TODO: Allow this to be a hostname, which is resolved to an address. */
+            /* TODO: Split this into a small function. */
+            struct dcc_allow_list *new;
+            new = malloc(sizeof *new);
+            if (!new)
+                rs_fatal("malloc failed");
+            new->next = opt_allowed;
+            opt_allowed = new;
+            if ((exitcode = dcc_parse_mask(poptGetOptArg(po), &new->addr, &new->mask)))
+                goto out_exit;
+        }
+            break;
+
         case 'n':
             if (arg_nchildren < 1 || arg_nchildren > 20000) {
                 rs_log_error("--tasks argument must be >1");
@@ -182,7 +202,7 @@ int distccd_parse_options(int argc, const char **argv)
         default:                /* bad? */
             rs_log(RS_LOG_NONAME|RS_LOG_ERR|RS_LOG_NO_PID, "%s: %s",
                    poptBadOption(po, POPT_BADOPTION_NOALIAS),
-                   poptStrerror(rc));
+                   poptStrerror(po_err));
             exitcode = EXIT_BAD_ARGUMENTS;
             goto out_exit;
         }
