@@ -1,7 +1,7 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*- 
  *
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/filename.c,v 1.6 2002/09/12 10:43:17 mbp Exp $ 
+ * $Header: /data/cvs/distcc/src/filename.c,v 1.7 2003/02/22 01:41:31 mbp Exp $ 
  *
  * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
  *
@@ -37,6 +37,7 @@
 #include "trace.h"
 #include "util.h"
 #include "exitcode.h"
+#include "filename.h"
 
 
 /*
@@ -95,7 +96,7 @@ static int dcc_set_file_extension(const char *sfile,
  * @returns preprocessed extension, (e.g. ".i"), or NULL if
  * unrecognized.
  **/
-char * dcc_preproc_exten(const char *e)
+const char * dcc_preproc_exten(const char *e)
 {
     if (e[0] != '.')
         return NULL;
