@@ -1,7 +1,7 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78 -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/io.c,v 1.49 2003/01/27 11:58:31 mbp Exp $ 
+ * $Header: /data/cvs/distcc/src/io.c,v 1.51 2003/02/24 05:12:37 mbp Exp $ 
  *
  * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
@@ -55,11 +55,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <assert.h>
 
 #include <sys/stat.h>
 #ifdef HAVE_SYS_SENDFILE_H
@@ -173,7 +171,8 @@ int dcc_r_str_alloc(int fd, int l, char **buf)
 {
      char *s;
 
-     assert(l > 0);
+     if (l < 0)
+         rs_fatal("oops, l < 0");
 
 /*      rs_trace("read %d byte string", l); */
 
