@@ -1,9 +1,8 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Id: lock.h,v 1.4 2002/10/09 14:33:27 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,5 +20,8 @@
  * USA
  */
 
-int dcc_lock_host(const struct dcc_hostdef *host, int slot, int block);
-int dcc_lock_local(const char *what);
+int dcc_lock_host(const char *lockname,
+                  const struct dcc_hostdef *host, int slot, int block,
+                  int *lock_fd);
+
+int dcc_unlock(int lock_fd);
