@@ -1,5 +1,22 @@
 #! /usr/bin/env python2.2
 
+# Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; either version 2 of the
+# License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+# USA
+
 """distcc test suite, using comfychair
 
 This script is called with $PATH pointing to the appropriate location
@@ -22,6 +39,7 @@ Copyright (C) 2002, 2003 by Martin Pool
 # abstract superclasses just provide methods that can be called,
 # rather than establishing default behaviour.
 
+# TODO: If the server fails with error 102, then try a different port.
 
 # TODO: Run the server in a different directory from the clients
 
@@ -130,11 +148,17 @@ Copyright (C) 2002, 2003 by Martin Pool
 # TODO: Test giving up privilege using --user.  Difficult -- we may
 # need root privileges to run meaningful tests.
 
-import time, sys, string, os, types, unittest, re, popen2, pprint
+# TODO: Test that recursion safeguard works.
+
+# TODO: Test using h_parsemask
+
+# TODO: Check using daemon --allow option for both allowed and denied
+# cases.  Check with multiple addresses.  Perhaps count only on
+# 127.0.0.1 - this ought to be enough.
+
+import time, sys, string, os, types, re, popen2, pprint
 import signal, os.path, string
 import comfychair
-from basecases import WithDaemon_Case, SimpleDistCC_Case, RunCmd_Case
-from ScanArgs import ScanArgs_Case
 
 from stat import *                      # this is safe
 
@@ -148,12 +172,129 @@ EXIT_BAD_HOSTSPEC            = 106
 EXIT_COMPILER_MISSING        = 110
 
 
+class SimpleDistCC_Case(comfychair.TestCase):
+    '''Abstract base class for distcc tests'''
+    def __init__(self):
+        self.cmd_log = ''
+
+    def setUp(self):
+        self.enterRundir()
+        self.stripEnvironment()
+
+    def enterRundir(self):
+        """Create and enter a temporary directory for this test."""
+        self.basedir = os.getcwd()
+        self.rundir = os.path.join(self.basedir,
+                                   'testtmp', 
+                                   self.__class__.__name__)
+        self.tmpdir = os.path.join(self.rundir, 'tmp')
+        os.system("rm -fr %s" % self.rundir)
+        os.makedirs(self.tmpdir)
+        os.system("mkdir -p %s" % self.rundir)
+        os.chdir(self.rundir)
+
+    def stripEnvironment(self):
+        """Remove all DISTCC variables from the environment, so that
+        the test is not affected by the development environment."""
+        for key in os.environ.keys():
+            if key[:7] == 'DISTCC_':
+                # NOTE: This only works properly on Python 2.2: on
+                # earlier versions, it does not call unsetenv() and so
+                # subprocesses may get confused.
+                del os.environ[key]
+        os.environ['TMPDIR'] = self.tmpdir
+
+    def tearDown(self):
+        self.leaveRundir()
+
+    def leaveRundir(self):
+        os.chdir(self.basedir)
+
+    def explainFailure(self, exc_info):
+        import traceback
+        print "-----------------------------------------------------------------"
+        traceback.print_exc(file=sys.stdout)
+        print self.cmd_log
+        print "-----------------------------------------------------------------"
+    
+    def fail(self, msg):
+        raise AssertionError(msg)
+
+    def assertEquals(self, a, b):
+        if not a == b:
+            raise AssertionError("assertEquals failed: %s" % `(a, b)`)
+            
+    def assertNotEqual(self, a, b):
+        if a == b:
+            raise AssertionError("assertNotEqual failed: %s" % `(a, b)`)
+
+    def assertReMatch(self, needle, haystack):
+        """Look for the regular expression re in haystack"""
+        if not re.search(needle, haystack):
+            self.fail("couldn't find re %s in %s" % (`needle`, `haystack`))
+            
+
+
+class WithDaemon_Case(SimpleDistCC_Case):
+    """Start the daemon, and then run a command locally against it.
+
+The daemon doesn't detach until it has bound the network interface, so
+as soon as that happens we can go ahead and start the client."""
+
+    def setUp(self):
+        SimpleDistCC_Case.setUp(self)
+        self.daemon_pidfile = os.path.join(os.getcwd(), "daemonpid.tmp")
+        self.daemon_logfile = os.path.join(os.getcwd(), "distccd.log")
+        self.server_port = 42000
+        self.startDaemon()
+        self.setupEnv()
+
+    def setupEnv(self):
+        os.environ['DISTCC_HOSTS'] = '127.0.0.1:%d' % self.server_port
+        os.environ['DISTCC_LOG'] = os.path.join(os.getcwd(), 'distcc.log')
+        os.environ['DISTCC_VERBOSE'] = '1'
+
+
+    def tearDown(self):
+        self.killDaemon()
+        SimpleDistCC_Case.tearDown(self)
+
+
+    def killDaemon(self):
+        import signal, time
+        pid = int(open(self.daemon_pidfile, 'rt').read())
+        os.kill(pid, signal.SIGTERM)
+
+        # We can't wait on it, because it detached.  So just keep
+        # pinging until it goes away.
+        while 1:
+            try:
+                os.kill(pid, 0)
+            except OSError:
+                break
+            time.sleep(0.2)
+
+
+    def startDaemon(self):
+        """Start a daemon in the background, return its pid"""
+        # The daemon detaches once it has successfully bound the
+        # socket, so if something goes wrong at startup we ought to
+        # find out straight away.  If it starts successfully, then we
+        # can go ahead and try to connect.
+        
+        cmd = ("distccd --verbose --daemon --log-file %s --pid-file %s --port %d"
+               % (self.daemon_logfile, self.daemon_pidfile, self.server_port))
+        self.runCmd(cmd)
+
+
+
+
 class StartStopDaemon_Case(WithDaemon_Case):
     def runTest(self):
         pass
 
 
-class VersionOption_Case(RunCmd_Case):
+class VersionOption_Case(SimpleDistCC_Case):
     """Test that --version returns some kind of version string.
 
     This is also a good test that the programs were built properly and are
@@ -168,7 +309,7 @@ class VersionOption_Case(RunCmd_Case):
                                  % prog, line1)
 
 
-class HelpOption_Case(RunCmd_Case):
+class HelpOption_Case(SimpleDistCC_Case):
     """Test --help is reasonable."""
     def runTest(self):
         for prog in 'distcc', 'distccd':
@@ -176,7 +317,7 @@ class HelpOption_Case(RunCmd_Case):
             self.assert_regexp("Usage:", out)
 
 
-class BogusOption_Case(RunCmd_Case):
+class BogusOption_Case(SimpleDistCC_Case):
     """Test handling of --bogus-option.
 
     Now that we support implicit compilers, this is passed to gcc, which returns 1."""
@@ -185,7 +326,7 @@ class BogusOption_Case(RunCmd_Case):
         self.runCmd("distccd --bogus-option", EXIT_BAD_ARGUMENTS)
 
 
-class GccOptionsPassed_Case(RunCmd_Case):
+class GccOptionsPassed_Case(SimpleDistCC_Case):
     """Test that options following the compiler name are passed to the compiler."""
     def runTest(self):
         out = self.runCmd("DISTCC_HOSTS=localhost distcc gcc --help")
@@ -194,7 +335,7 @@ class GccOptionsPassed_Case(RunCmd_Case):
         self.assert_re_match(r"^Usage: gcc", out)
 
 
-class StripArgs_Case(RunCmd_Case):
+class StripArgs_Case(SimpleDistCC_Case):
     """Test -D and -I arguments are removed"""
     def runTest(self):
         cases = (("gcc -c hello.c", "gcc -c hello.c"),
@@ -221,7 +362,7 @@ class StripArgs_Case(RunCmd_Case):
             self.assertEquals(o, expect)
 
 
-class IsSource_Case(RunCmd_Case):
+class IsSource_Case(SimpleDistCC_Case):
     def runTest(self):
         """Test distcc's method for working out whether a file is source"""
         cases = (( "hello.c",          "source",       "not-preprocessed" ),
@@ -242,6 +383,53 @@ class IsSource_Case(RunCmd_Case):
 
 
 
+class ScanArgs_Case(SimpleDistCC_Case):
+    '''Test understanding of gcc command lines'''
+    def runTest(self):
+        cases = [("gcc -c hello.c", "distribute", "hello.c", "hello.o"),
+                 ("gcc hello.c", "local"),
+                 ("gcc -o /tmp/hello.o -c ../src/hello.c", "distribute", "../src/hello.c", "/tmp/hello.o"),
+                 ("gcc -DMYNAME=quasibar.c bar.c -c -o bar.o", "distribute", "bar.c", "bar.o"),
+                 ("gcc -ohello.o -c hello.c", "distribute", "hello.c", "hello.o"),
+                 ("ccache gcc -c hello.c", "distribute", "hello.c", "hello.o"),
+                 ("gcc hello.o", "local"),
+                 ("gcc -o hello.o hello.c", "local"),
+                 ("gcc -o hello.o -c hello.s", "local"),
+                 ("gcc -o hello.o -c hello.S", "local"),
+                 ("gcc -fprofile-arcs -ftest-coverage -c hello.c", "local", "hello.c", "hello.o"),
+                 ("gcc -S hello.c", "distribute", "hello.c", "hello.s"),
+                 ("gcc -c -S hello.c", "distribute", "hello.c", "hello.s"),
+                 ("gcc -S -c hello.c", "distribute", "hello.c", "hello.s"),
+                 ("gcc -M hello.c", "local"),
+                 ("gcc -ME hello.c", "local"),
+                 
+                 ("gcc -MD -c hello.c", "distribute", "hello.c", "hello.o"),
+                 ("gcc -MMD -c hello.c", "distribute", "hello.c", "hello.o"),
+
+                 # Assemble to stdout (thanks Alexandre).  
+                 ("gcc -S foo.c -o -", "local"),
+                 ("-S -o - foo.c", "local"),
+                 ("-c -S -o - foo.c", "local"),
+                 ("-S -c -o - foo.c", "local"),
+                 ]
+        for tup in cases:
+            self.checkScanArgs(*tup)
+
+    def checkScanArgs(self, ccmd, mode, input=None, output=None):
+        o = self.runCmd("h_scanargs %s" % ccmd)
+        o = o[:-1]                      # trim \n
+        os = string.split(o)
+        if mode != os[0]:
+            self.fail("h_scanargs %s gave %s mode, expected %s" %
+                      (ccmd, os[0], mode))
+        if mode == 'distribute':
+            if os[1] <> input:
+                self.fail("h_scanargs %s gave %s input, expected %s" %
+                          (ccmd, os[1], input))
+            if os[2] <> output:
+                self.fail("h_scanargs %s gave %s output, expected %s" %
+                          (ccmd, os[2], output))
+
 
 
 class ImplicitCompilerScan_Case(ScanArgs_Case):
@@ -252,10 +440,12 @@ class ImplicitCompilerScan_Case(ScanArgs_Case):
                  ("-o hello.o -c hello.c", "distribute", "hello.c", "hello.o"),
                  ]
         for tup in cases:
-            self.checkScanArgs(*tup)
+            # NB use "apply" rather than new syntax for compatibility with
+            # venerable Pythons.
+            apply(self.checkScanArgs, tup)
             
 
-class ExtractExtension_Case(RunCmd_Case):
+class ExtractExtension_Case(SimpleDistCC_Case):
     def runTest(self):
         """Test extracting extensions from filenames"""
         for f, e in (("hello.c", ".c"),
@@ -267,7 +457,7 @@ class ExtractExtension_Case(RunCmd_Case):
             assert out == e
 
 
-class DaemonBadPort_Case(RunCmd_Case):
+class DaemonBadPort_Case(SimpleDistCC_Case):
     def runTest(self):
         """Test daemon invoked with invalid port number"""
         self.runCmd("distccd --log-file=distccd.log --port 80000",
@@ -275,7 +465,7 @@ class DaemonBadPort_Case(RunCmd_Case):
         self.assert_no_file("daemonpid.tmp")
 
 
-class InvalidHostSpec_Case(RunCmd_Case):
+class InvalidHostSpec_Case(SimpleDistCC_Case):
     def runTest(self):
         """Test various invalid DISTCC_HOSTS
         
@@ -285,7 +475,7 @@ class InvalidHostSpec_Case(RunCmd_Case):
                          EXIT_BAD_HOSTSPEC)
 
 
-class ParseHostSpec_Case(RunCmd_Case):
+class ParseHostSpec_Case(SimpleDistCC_Case):
     def runTest(self):
         """Check operation of dcc_parse_hosts_env.
 
@@ -296,7 +486,7 @@ class ParseHostSpec_Case(RunCmd_Case):
         ipv4-localhost
         """
         expected="""7
-   4 LOCAL
+   1 LOCAL
    4 TCP 127.0.0.1 3632
    4 SSH (no-user) angry (no-command)
    4 SSH ted angry (no-command)
@@ -328,8 +518,8 @@ class Compilation_Case(WithDaemon_Case):
         return "testtmp.c"              # default
     
     def compile(self):
-        msgs = self.runCmd(self.compileCmd())
-        msgs += self.runCmd(self.linkCmd())
+        msgs = (self.runCmd(self.compileCmd()) +
+                self.runCmd(self.linkCmd()))
         self.checkCompileMsgs(msgs)
 
     def compileCmd(self):
@@ -457,6 +647,7 @@ class NoDetachDaemon_Case(CompileHello_Case):
         pid, ret = os.wait()
         self.assertEquals(self.pid, pid)
         self.assertEquals(ret, 0)
+
         
 
 class ImplicitCompiler_Case(CompileHello_Case):
@@ -761,54 +952,75 @@ class ModeBits_Case(CompileHello_Case):
         self.assertEquals(S_IMODE(os.stat("testtmp.o")[ST_MODE]), 0666)
 
 
-def test_unresolvable_server():
-    """Test handling of an unresolvable host"""
+class CheckRoot_Case(comfychair.TestCase):
+    """Stub case that checks this is run by root.  Not used by default."""
+    def setUp(self):
+        self.require_root()
+
+
+class EmptySource_Case(Compilation_Case):
+    """Check compilation of empty source file
+
+    It must be treated as preprocessed source, otherwise cpp will
+    insert a # line, which will give a false pass.  """
+    
+    def source(self):
+        return ''
+
+    def runTest(self):
+        self.compile()
+
+    def compile(self):
+        self.runCmd("distcc cc -c %s" % self.sourceFilename())
+
+    def sourceFilename(self):
+        return "testtmp.i"
+
 
 
-def test_unreachable_server():
-    """Test handling of a host that (probably) can't be reached"""
 
 
 if __name__ == '__main__':
-    import EmptySource
+    import sys
     
     if len(sys.argv) > 1:
         fns = map(locals().get, sys.argv[1:])
-        comfychair.runtests(fns)
+        ret = comfychair.runtests(fns)
     else:
-        comfychair.runtests([ScanArgs_Case,
-                             ImplicitCompilerScan_Case,
-                             StripArgs_Case,
-                             StartStopDaemon_Case,
-                             CompileHello_Case,
-                             CppError_Case,
-                             BadInclude_Case,
-                             PreprocessPlainText_Case,
-                             NoDetachDaemon_Case,
-                             SBeatsC_Case,
-                             DashD_Case,
-                             BinFalse_Case,
-                             BinTrue_Case,
-                             VersionOption_Case,
-                             HelpOption_Case,
-                             BogusOption_Case,
-                             MultipleCompile_Case,
-                             GccOptionsPassed_Case,
-                             IsSource_Case,
-                             ExtractExtension_Case,
-                             ImplicitCompiler_Case,
-                             DaemonBadPort_Case,
-                             NoServer_Case,
-                             InvalidHostSpec_Case,
-                             ParseHostSpec_Case,
-                             ImpliedOutput_Case,
-                             SyntaxError_Case,
-                             NoHosts_Case,
-                             MissingCompiler_Case,
-                             RemoteAssemble_Case,
-                             PreprocessAsm_Case,
-                             ModeBits_Case,
-                             EmptySource.EmptySource_Case,
-                             Concurrent_Case,
-                             ThousandFold_Case,
-                             BigAssFile_Case])
+        ret = comfychair.runtests([ScanArgs_Case,
+                                   ImplicitCompilerScan_Case,
+                                   StripArgs_Case,
+                                   StartStopDaemon_Case,
+                                   CompileHello_Case,
+                                   CppError_Case,
+                                   BadInclude_Case,
+                                   PreprocessPlainText_Case,
+                                   NoDetachDaemon_Case,
+                                   SBeatsC_Case,
+                                   DashD_Case,
+                                   BinFalse_Case,
+                                   BinTrue_Case,
+                                   VersionOption_Case,
+                                   HelpOption_Case,
+                                   BogusOption_Case,
+                                   MultipleCompile_Case,
+                                   GccOptionsPassed_Case,
+                                   IsSource_Case,
+                                   ExtractExtension_Case,
+                                   ImplicitCompiler_Case,
+                                   DaemonBadPort_Case,
+                                   NoServer_Case,
+                                   InvalidHostSpec_Case,
+                                   ParseHostSpec_Case,
+                                   ImpliedOutput_Case,
+                                   SyntaxError_Case,
+                                   NoHosts_Case,
+                                   MissingCompiler_Case,
+                                   RemoteAssemble_Case,
+                                   PreprocessAsm_Case,
+                                   ModeBits_Case,
+                                   EmptySource_Case,
+                                   Concurrent_Case,
+                                   ThousandFold_Case,
+                                   BigAssFile_Case])
+    sys.exit(ret)
